/*
 * Jsonix is a JavaScript library which allows you to convert between XML
 * and JavaScript object structures.
 *
 * Copyright (c) 2010, Aleksei Valikov, Highsource.org
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Aleksei Valikov nor the
 *       names of contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ALEKSEI VALIKOV BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

Jsonix.Model.ElementsPropertyInfo = Jsonix.Class(Jsonix.Model.AbstractElementsPropertyInfo, {
	elementTypeInfos : null,
	elementTypeInfosMap : null,
	initialize : function(options) {
		Jsonix.Util.Ensure.ensureObject(options);
		Jsonix.Model.AbstractElementsPropertyInfo.prototype.initialize.apply(this, [ options ]);
		// TODO Ensure correct arguments
		Jsonix.Util.Ensure.ensureArray(options.elementTypeInfos);
		this.elementTypeInfos = options.elementTypeInfos;
		this.elementTypeInfosMap = {};
		for ( var index = 0; index < this.elementTypeInfos.length; index++) {
			var elementTypeInfo = this.elementTypeInfos[index];
			Jsonix.Util.Ensure.ensureObject(elementTypeInfo);
			if (Jsonix.Util.Type.isObject(elementTypeInfo.elementName)) {
				Jsonix.Util.Ensure.ensureString(elementTypeInfo.elementName.localPart, 'Element name must contain a string property [localPart].');
				elementTypeInfo.elementName = Jsonix.XML.QName.fromObject(elementTypeInfo.elementName);
			} else {
				Jsonix.Util.Ensure.ensureString(elementTypeInfo.elementName);
				elementTypeInfo.elementName = new Jsonix.XML.QName(this.defaultElementNamespaceURI, elementTypeInfo.elementName);
			}
			this.elementTypeInfosMap[elementTypeInfo.elementName.key] = elementTypeInfo.typeInfo;
		}
	},
	unmarshalElement : function(context, input, callback) {
		// TODO make sure it's the right event type
		var elementName = input.getName();
		var key = elementName.key;
		var typeInfo = this.elementTypeInfosMap[key];
		if (Jsonix.Util.Type.exists(typeInfo)) {
			return callback(typeInfo.unmarshal(context, input));
		}
		// TODO better exception
		throw "Element [" + key + "] is not known in this context";
	},
	marshalElement : function(context, value, output) {
		for ( var index = 0; index < this.elementTypeInfos.length; index++) {
			var elementTypeInfo = this.elementTypeInfos[index];
			var typeInfo = elementTypeInfo.typeInfo;
			if (typeInfo.isInstance(value)) {
				var elementName = elementTypeInfo.elementName;
				this.marshalElementTypeInfo(context, value, elementName, typeInfo, output);
				return;
			}
		}
		throw "Could not find an element with type info supporting the value [" + value + "].";
	},
	buildStructureElements : function(context, structure) {
		for ( var index = 0; index < this.elementTypeInfos.length; index++) {
			var elementTypeInfo = this.elementTypeInfos[index];
			structure.elements[elementTypeInfo.elementName.key] = this;
		}
	},
	CLASS_NAME : 'Jsonix.Model.ElementsPropertyInfo'
});
